/*  Coman
 *  Copyright (C) 2000 Luc Lavoie <bart98_@yahoo.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <string.h>
#include <stdio.h> 
#include <stdlib.h>
#include "coman.h"
#include "libcoman.h"

CConfig *FirstConfig;

int InitConfig (char *FileName)
{
	CConfig *New;
	FILE *Config;
	char *GroupName, *SettingName, *ValueName, *tmp;
	char c;
	int i = 0;

	Config = fopen(FileName, "rt");
	if (Config == NULL) {
		printf("Could not load settings in file %s.\n", FileName);
		return -1; }
  	else {
		New = NewConfig(GetTotalConfigs()+1);
  		while (New->State != EOF) {
			tmp = GetLine(Config);
			if (tmp == NULL)
				continue;
			New->Pos = 0;
			if (tmp[0] == '[') {
				GroupName = ParseToDelimeter(&tmp[1], ']');
				if (GroupName == NULL)
					printf("Parse error in group definition.\n");
				else {
					InitGroup(New, GroupName);
					New->CurrentGroup = New->TotalGroups-1;
					free(GroupName); }
				continue; }
			SettingName = ParseToDelimeter(tmp, '=');
			if (SettingName == NULL)
				printf("Parse error in setting definition.\n");
			else {
				ValueName = ParseToDelimeter(&tmp[New->Pos+1], '\0');
				InitSetting(New, SettingName, ValueName);
				free(ValueName);
				free(SettingName); } }
	    New->CurrentGroup = -1;
    	fclose(Config);
    	return New->Id; }
}

int CloseConfig (char *FileName, int config)
{
	CConfig *Ptr;
	FILE *Config;
	int i = 0, j = 0;

	Config = fopen(FileName, "w");
	if (!Config) {
        printf("Could not save settings to file %s\n", FileName);
        return 0; }
	else {
      	fprintf(Config, "# Settings file generated by Coman.\n\n");
		Ptr = GetConfig(config);
		if (Ptr == NULL)
			return 0;
        while (i < Ptr->TotalGroups) {
	        j = 0;
		    fprintf(Config, "[%s]\n", Ptr->Groups[i].Name);
	        while (j < Ptr->Groups[i].TotalSettings) {
		        fprintf(Config, "%s=%s\n", Ptr->Groups[i].Settings[j].Name, 
				Ptr->Groups[i].Settings[j].Value);
				j++; }
		    fprintf(Config, "\n");
			i++; }
      	fclose(Config);
		RemoveConfig(Ptr->Id);
		return 1;
	}
}

void SetGroup (char *GroupName, int config)
{
	CConfig *Ptr;	
	int i = 0;

	Ptr = GetConfig(config);
	if (Ptr == NULL)
		return;
	while (i < Ptr->TotalGroups) {
		if (!strcmp(GroupName, Ptr->Groups[i].Name)) {
			Ptr->CurrentGroup = i;
			return; }
		i++; }
	InitGroup(Ptr, GroupName);
	Ptr->CurrentGroup = Ptr->TotalGroups-1;
}

char* GetString (char *SettingName, int config)
{
	CConfig *Ptr;	
	int i = 0;

	Ptr = GetConfig(config);
	if (Ptr == NULL)
		return;
	if (Ptr->CurrentGroup == -1)
		return NULL;
	while (i < Ptr->Groups[Ptr->CurrentGroup].TotalSettings) {
		if (!strcmp(SettingName, 
            Ptr->Groups[Ptr->CurrentGroup].Settings[i].Name))
			return Ptr->Groups[Ptr->CurrentGroup].Settings[i].Value;
		i++; }
	return NULL;
}

int GetNumber (char *SettingName, int config)
{
	char *tmp;

	tmp = GetString(SettingName, config);
	if (tmp == NULL)
		return 0;
	return atoi(tmp);
}

int WriteString (char *SettingName, char *Value, int config)
{
	CConfig *Ptr;	
	int i = 0;

	Ptr = GetConfig(config);
	if (Ptr == NULL)
		return;
	if (Ptr->CurrentGroup == -1)
		return 0;
	while (i < Ptr->Groups[Ptr->CurrentGroup].TotalSettings) {
		if (!strcmp(SettingName, 
        Ptr->Groups[Ptr->CurrentGroup].Settings[i].Name)) {
			free(Ptr->Groups[Ptr->CurrentGroup].Settings[i].Value);
			Ptr->Groups[Ptr->CurrentGroup].Settings[i].Value = 
			strdup(Value);
			return 1; }
		i++; }
	InitSetting(Ptr, SettingName, Value);
	return 1;
}

int WriteNumber (char *SettingName, int Value, int config)
{
	char tmp[100];

	sprintf(tmp, "%d", Value);
	if (!WriteString(SettingName, tmp, config))
		return 0;
	else
		return 1;
}

/*
 internal functions
*/

char* ParseToDelimeter (char *str, char Delimeter)
{
	CConfig *Ptr;
	char Tmp[100];
	char c;
	int i = 0;

 	Ptr = GetConfig(GetTotalConfigs());
	c = str[i];
	while(c != Delimeter && c != '\n' && c != EOF) {
		Tmp[i] = c;
		i++;
		c = str[i]; }
	if (c == EOF)
		Ptr->State = EOF;
	if (c == '\n')
		return NULL;
	Ptr->Pos = i;
	Tmp[i] = '\0';
	return strdup(Tmp);
}

char* GetLine (FILE *file)
{
	CConfig *Ptr;
	char c, tmp[100];
	int i = 0;

	Ptr = GetConfig(GetTotalConfigs());
	c = fgetc(file);
	while (c != '\n' && c != EOF) {
		if (c == '#') {
			while (c != '\n')
				c = fgetc(file);
			if (i == 0)
				return NULL;
			else
				return strdup(tmp); }
		tmp[i] = c;
		i++;
		c = fgetc(file); }
	if (c == EOF)
		Ptr->State = EOF;
	if (i == 0)
		return NULL;
	tmp[i] = '\0';
	return strdup(tmp);
}

void InitGroup (CConfig *Config, char *GroupName)
{
	Config->TotalGroups++;
	Config->Groups = realloc(Config->Groups,
    Config->TotalGroups*sizeof(CGroup));
	Config->Groups[Config->TotalGroups-1].TotalSettings = 0;
	Config->Groups[Config->TotalGroups-1].Settings = NULL;
	Config->Groups[Config->TotalGroups-1].Name = strdup(GroupName);
}

void InitSetting (CConfig *Config, char *SettingName, char *Value)
{
	Config->Groups[Config->CurrentGroup].TotalSettings++;
	Config->Groups[Config->CurrentGroup].Settings = 
    realloc(Config->Groups[Config->CurrentGroup].Settings,
	Config->Groups[Config->CurrentGroup].TotalSettings*sizeof(CSetting));
	Config->Groups[Config->CurrentGroup].
    Settings[Config->Groups[Config->CurrentGroup].TotalSettings-1].Name = 
	strdup(SettingName);	
	Config->Groups[Config->CurrentGroup].
    Settings[Config->Groups[Config->CurrentGroup].TotalSettings-1].Value = 
	strdup(Value);
}

CConfig* NewConfig (int Id)
{
	CConfig *config;

	if (GetTotalConfigs() == 0)	{
		FirstConfig = NULL;
		FirstConfig = calloc(1, sizeof(CConfig));
		FirstConfig->Next = NULL;
		FirstConfig->Id = Id;
		FirstConfig->State = 0;
		FirstConfig->TotalGroups = 0;
		FirstConfig->Groups = NULL;
		return FirstConfig;	}
	else {
		for(config = FirstConfig; config->Next != NULL; config = config->Next);
		config->Next = calloc(1, sizeof(CConfig));
		config->Next->Next = NULL;
		config->Next->Id = Id;
		config->Next->State = 0;
		config->Next->TotalGroups = 0;
		config->Next->Groups = NULL;
		return config->Next; }	
}

CConfig* GetConfig(int Id)
{
	CConfig *config;

	for(config = FirstConfig; config != NULL; config = config->Next) {
		if (config->Id == Id)
			return config; }
	return NULL;
}

void RemoveConfig (int Id)
{
	CConfig *Ptr, *Victim;

	Victim = GetConfig(Id);
	if (Victim == NULL)
		return;
	if (Victim == FirstConfig) {
		FirstConfig = FirstConfig->Next;
		free(Victim);
		return; }
	for(Ptr = FirstConfig; Ptr->Next != Victim; Ptr = Ptr->Next);
 	Ptr->Next = Victim->Next;
  	free(Victim);
}

int GetTotalConfigs (void)
{
	CConfig *Ptr;
	int i = 0;	

	for(Ptr = FirstConfig; Ptr != NULL; Ptr = Ptr->Next)
		i++;
	return i;
}
